#ifndef __PFE_DEF_WORDS_H
#define __PFE_DEF_WORDS_H "%full_filespec: def-words.h~5.0 %"

/** 
 * -- types, codes and macros for symbol table load
 *
 *  Copyright (C) Tektronix, Inc. 1998 - 2000. All rights reserved.
 *
 *  @see     GNU LGPL
 *  @author  Tektronix CTE             @(#) %derived_by: guidod %
 *  @version %version: 5.0 %
 *    (%date_modified: Mon May 15 18:00:58 2000 %)
 *
 *  @description
 *         here are the macros that are used to build the various
 *         symbols tables, including wordsets and loadlists.
 */
/*@{*/

#include <pfe/incl-ext.h>
#include <pfe/def-macro.h>
#include <pfe/def-const.h>

#define P4WORDS(SET)  P4CAT (SET,_LTX_p4_WORDS)
#define P4WLIST(SET)  P4CAT (SET,_LTX_p4_WLIST)
#define P4LLIST(SET)  P4CAT (SET,_LTX_p4_LLIST)

#define extern_P4WORDS(SET) extern const p4Words P4WORDS(SET)

typedef struct			/* describe a word for registration */
{				/* in the dictionary */
    const char *name;		/* name of word */
    p4code ptr;			/* code it executes or something */
}				/* according to spc@pineal.math.fau.edu: */
p4Word;				/* 'void *' doesn't work here on SGI */

typedef struct			/* describes the set of words */
{				/* a module loads into the dictionary */
    int n;			/* how many words? */
    const p4Word *w;		/* pointer to vector of words */
    const char *name;		/* name of word set, or null */
}
p4Words;

#define P4_LISTWORDS( SET )  \
    static const p4Word P4WLIST(SET)[]

# define P4_COUNTWORDS(SET,NAME) \
    const p4Words* P4LLIST(SET)() \
    {	  			 \
        extern const p4Words P4WORDS(SET); \
        return &P4WORDS(SET);  \
    }                            \
    const p4Words P4WORDS(SET) = \
    {				\
        DIM (P4WLIST (SET)),	\
        P4WLIST (SET),		\
        NAME			\
    }
  
#ifdef _P4_SOURCE

#define LISTWORDS( SET )  \
    static const p4Word P4WLIST(SET)[]

# define COUNTWORDS(SET,NAME) \
    const p4Words P4WORDS(SET) = \
    {				\
        DIM (P4WLIST (SET)),	\
        P4WLIST (SET),		\
        NAME			\
    }
  
#endif  

/* Encoding the kind of definition i.e. which runtime to fill into the cfa. 
 * 
 * the original pfe had 3 (or 4?) levels of load-tables.
 * the definitions above define a two level approach.
 * the following definitions define a one level approach,
 * and up to now, no heritage-defines are given.
 *
 * each uppercase-name is an immediate word, all others are not.
 */
 
#define p4_CO 	'p' 	/* CO */ /* ordinary primitive (code) */
#define p4_CI   'P' 	/* CI */ 
#define p4_CS   'X'   	/* CS */ /* smart-word (semant) */
#define p4_CX   'x'     /* CX */ /* auto-init */
 
#define p4_OV   'v'     /* OV */ /* ordinary variable */
#define p4_IV   'V'     /* IV */
#define p4_OC   'c'     /* OC */ /* ordinary constant */
#define p4_IC   'C'    	/* IC */
#define p4_OL   'l'     /* OL */ /* ordinary value */
#define p4_IL   'L'     /* IL */

#define p4_DV   'm'     /* DV */ /* dict variable (threaded) */
#define p4_DC   'n'     /* DC */ /* dict constant (threaded) */
#define p4_OT   'o'     /* offset word */

#define p4_OS   's'     /* open slot */
#define p4_IS   's'     /* init slot */

#define p4_IN	'W'  	/* make sure the vocabulary exists, and put all */
    			/* of the following words into it */
#define p4_ON 	'w'     /* make sure the voc is in the default search order */ 
                        /* and put all of the referenced wordset into it */

/* macros to build entries in the wordlists: 
 * until all sematic-words have a proper name along, we need to help
 * the decompiler here to print the name. Since PFE uses %.*s to print,
 * it is okay to the upper bound of the length as a dummy count byte,
 * which is 0xFF for immediate words and 0x9F for the others.
 */

#define	P4_CO( NM, PCODE)	{ "p\x9F"NM, &P4CODE (PCODE) }
#define	P4_CI( NM, PCODE)	{ "P\xFF"NM, &P4CODE (PCODE) }
#define P4_CS( NM, SEM)		{ "X\xFF"NM, (p4code)&P4SEMANTICS(SEM) }
#define P4_CX( NM, PCODE)	{ "x\x9F"NM, &P4CODE (PCODE) } /* AUTO-INIT */

#define P4_OV( NM)		{ "v\x9F"NM, (p4code)0) }
#define	P4_OC( NM, VAL)		{ "c\x9F"NM, (p4code)(VAL) }
#define P4_OL( NM, VAL)		{ "l\x9F"NM, (p4code)(VAL) }
#define P4_OT( NM, VAL)   	{ "o\x9F"NM, (p4code)(VAL) }

#define P4_IV( NM)		{ "V\xFF"NM, (p4code)0) }
#define	P4_IC( NM, VAL)		{ "C\xFF"NM, (p4code)(VAL) }
#define P4_IL( NM, VAL)		{ "L\xFF"NM, (p4code)(VAL) }

/* #define	VO(NM,LIST)	{ "\112"NM, (p4code)(&P4WORDS(LIST,WList)) } */

#define	P4_DV( NM, MEMBER)	{ "m\x9F"NM, (p4code)OFFSET_OF(p4_Thread, MEMBER) }
#define	P4_DC( NM, MEMBER)	{ "n\x9F"NM, (p4code)OFFSET_OF(p4_Thread, MEMBER) }

#define P4_OT( NM, SIZE)  	{ "o\x9F"NM, (p4code)(SIZE) }

#define P4_OS( NM, SLOTVAR)  	{ "z\x9F"NM, (p4code)(SLOTVAR) }
#define P4_IS( NM, SIZE)        { "Z\xFF"NM, (p4code)(SIZE) }

#define P4_IN( NM, DESCR)       { "W\xFF"NM, (p4code)(DESCR) } /* DESCR may be null */
#define P4_ON( NM, REFER)  	{ "w\x9F"NM, (p4code)(REFER) } /* REFER may be null */

#ifdef _P4_SOURCE
/* macros to build entries in the wordlists: */

#define	CO(NM,PCODE)	{ "p\x9F"NM, &P4CODE (PCODE) }
#define	CI(NM,PCODE)	{ "P\xFF"NM, &P4CODE (PCODE) }
#define CS(NM,SEM)	{ "X\xFF"NM, (p4code)&P4SEMANTICS(SEM) }
#define CX(NM,PCODE)	{ "x\x9F"NM, &P4CODE (PCODE) } /* AUTO-INIT */

#define OV(NM)		{ "v\x9F"NM, (p4code)0) }
#define	OC(NM,VAL)	{ "c\x9F"NM, (p4code)(VAL) }
#define OL(NM,VAL)	{ "l\x9F"NM, (p4code)(VAL) }
#define OT(NM,VAL)      { "o\x9F"NM, (p4code)(VAL) }

#define IV(NM)		{ "V\xFF"NM, (p4code)0) }
#define	IC(NM,VAL)	{ "C\xFF"NM, (p4code)(VAL) }
#define IL(NM,VAL)	{ "L\xFF"NM, (p4code)(VAL) }

/* #define VO(NM,LIST)   { "\112"NM, (p4code)(&P4WORDS(LIST,WList)) } */

# if __IBMC__ && __OS2__	/* IBM C Set/2 thinks OFFSET_OF is no constant expr */
# define DV(NM,MEMBER)  { "m\x9F"NM, (p4code)do_##MEMBER }
# define DC(NM,MEMBER)  { "n\x9F"NM, (p4code)do_##MEMBER }
# else
# define DV(NM,MEMBER)  { "m\x9F"NM, (p4code)OFFSET_OF(p4_Thread, MEMBER) }
# define DC(NM,MEMBER)  { "n\x9F"NM, (p4code)OFFSET_OF(p4_Thread, MEMBER) }
# endif

#endif


/* ------------------------
   defining a load list
   ------------------------  */
   
#define P4_LOAD_END  ((void*)0)   
#define P4_LOAD_INTO ((void*)1)
#define P4_LOAD_EXEC ((void*)2)
#define P4_LOAD_ORDER ((void*)3)
#define P4_LOAD_SLOT ((void*)4)
#define P4_LOAD_MAX  ((void*)8)
   
#define P4_LOADSTRUCT(NAME) P4CAT3(p4_,NAME,_LoadList)
#define P4_LOADLIST(NAME) const void* P4_LOADSTRUCT(NAME) []
   
#ifdef MODULE        
#define P4_MODULE_LIST(name) \
        void* p4_LTX_p4_MODULE() \
        {               	\
           return &P4_LOADSTRUCT(name); \
        }
#else
#define P4_MODULE_LIST(name) \
        void* name##_LTX_p4_MODULE() \
        {               	\
           return &P4_LOADSTRUCT(name); \
        }
#endif


/*@}*/
#endif 

