#ifndef __PFE_INCL_SUP_H
#define __PFE_INCL_SUP_H "%full_filespec: incl-sup.h~5.3:incl:bln_12xx!1 %"

/** 
 * -- declare types and prototypes for support.c
 * 
 *  Copyright (C) Tektronix, Inc. 1998 - 2000. All rights reserved.
 *
 *  @see     GNU LGPL
 *  @author  Tektronix CTE             @(#) %derived_by: guidod %
 *  @version %version: 5.3 %
 *    (%date_modified: Mon Sep 18 18:31:04 2000 %)
 *  @description
 *            most of the routines of the internal forth system
 *            are declared here
 */
/*@{*/

#include <pfe/incl-ext.h>

#ifndef FILE
#include <stdio.h>
#endif

/* miscellaneous =========================================================== */

void p4_delay (int ms);
extern void p4_exit (int);

FCode (p4_rot);
FCode (p4_invert);
FCode (p4_fetch);
FCode (p4_pocket);
FCode (p4_update);
FCode (p4_save_buffers);
FCode (p4_close_all_files);
FCode (p4_decimal);
FCode (p4_key_question);

/* strings, memory ========================================================= */

p4cell p4_aligned (p4cell n);
p4cell p4_dfaligned (p4cell n);
void *p4_xcalloc (int n_elem, size_t size);
void *p4_calloc (int n_elem, size_t size);
void *p4_xalloc (size_t size);
void p4_xfree (void*);

#ifdef _P4_SOURCE
# if defined HAVE_STRNCASECMP 
# define p4_strncmpi strncasecmp
# elif defined HAVE_STRNICMP
# define p4_strncmpi strnicmp
# else
int p4_strncmpi ( const char *s1, const char* s2, int n); 
# endif
# ifndef strncmpi       /*cygwin32 has it already def'd*/
# define strncmpi p4_strncmpi
# endif
#endif

void p4_strpush (const char *s);
char *p4_pocket (void);
int p4_dash_trailing (char *s, int n);
void p4_lower (char *p, int n); 
void p4_upper (char *p, int n);
char *p4_store_c_string (const char *src, int n, char *dst, int max);
char *p4_store_filename (const char *src, int n, char *dst, int max);
char *p4_expand_filename (const char *nm, const char *path,
		       const char *ext, char *fn);
char *p4_search (const char *p1, int u1, const char *p2, int u2);
int p4_match (const char *pattern, const char *string, int ic);

/* floored divide and number i/o conversion ================================ */

udiv_t p4_udiv (p4ucell num, p4ucell denom);
fdiv_t p4_fdiv (p4cell num, p4cell denom);
p4ucell p4_u_d_div (p4udcell *ud, p4ucell denom);
void p4_u_d_mul (p4udcell *ud, p4ucell w, p4ucell c);

int p4_dig2num (p4char c, p4ucell *n, p4ucell base);
char p4_num2dig (p4ucell n);
void p4_hold (char c);
const char *p4_to_number (const char *p, p4ucell *n, p4udcell *d, p4ucell base);
int p4_number_question (const char *p, p4ucell n, p4dcell *d);

char *p4_str_ud_dot_r (p4udcell ud, char *p, int w, int base);
char *p4_str_d_dot_r (p4dcell d, char *p, int w, int base);
char *p4_str_dot (p4cell n, char *p, int base);

FCode (p4_less_sh);
FCode (p4_sh_s);
FCode (p4_sh_greater);

FCode (p4_s_to_d);			/* in core.c */
FCode (p4_d_dot_r);			/* in double.c */
FCode (p4_d_dot);			/* in double.c */
FCode (p4_dot_r);			/* in core.c */
FCode (p4_dot);			/* in core.c */

void p4_d_dot_r (p4dcell d, int w);
void p4_d_dot (p4dcell d);
void p4_dot_r (p4cell n, int w);
void p4_dot (p4cell n);

/* console i/o ============================================================= */

FCode (p4_standard_io);

void p4_outc (char c);		/* these work on C-strings */
void p4_outs (const char *s);	/* but adjust the OUT variable */
int p4_outf (const char *s, ...);
void p4_type_on_line (const char *s, p4cell n);
void p4_type (const char *s, p4cell n); /* this works with a Forth-string */

FCode (p4_type);			/* in core.c */
FCode (p4_space);			/* in core.c */
FCode (p4_cr);			/* in core.c */
FCode (p4_backspace);		/* in misc.c */
FCode (p4_bell);			/* in misc.c */

void p4_emits (int n, const char del);
FCode (p4_spaces);			/* in core.c */

void p4_tab (int n);
void p4_dot_line (p4_File *fid, p4cell blk, p4cell line);
int p4_expect (char *p, p4cell n);
int p4_accept (char *p, int n);
int p4_Q_stop (void);
int p4_Q_cr (void);
FCode (p4_start_Q_cr);	/* in misc.c */

int p4_err (int);

/* files =================================================================== */

long fsize (FILE *f);
long fn_size (const char *fn);
long fn_copy (const char *dst, const char *src, long limit);
int fn_move (const char *src, const char *dst);
int fn_resize (const char *fn, long size);

#ifndef HASNT_SYSTEM
int p4_systemf (const char *s, ...);
#endif /* HASNT_SYSTEM */

void p4_file_errorz (const char *fn);
void p4_file_error (const char *fn, int len);
int p4_file_access (const char *fn, int len);
p4_File *p4_create_file (const char *p, int n, int mode);
p4_File *p4_open_file (const char *p, int n, int mode);
int p4_close_file (p4_File *f);
int p4_reposition_file (p4_File *f, long pos);
int p4_read_file (void *p, p4ucell *n, p4_File *f);
int p4_write_file (void *p, p4ucell n, p4_File *f);
int p4_resize_file (p4_File *f, long size);
int p4_read_line (char *p, p4ucell *u, p4_File *f, p4cell *ior);

/* source input ============================================================ */

FCode (p4_query);
int p4_next_line (void);

p4_File *p4_open_block_file (const char *name, int len);
int p4_use_block_file (const char *name, int len);
void p4_read_write (p4_File *f, char *p, p4ucell n, int readflag);
char *p4_buffer (p4_File *fid, p4ucell n, int *reload);
char *p4_block (p4_File *fid, p4ucell n);
void p4_update (p4_File *fid);
void p4_empty_buffers (p4_File *fid);
void p4_save_buffers (p4_File *fid);
void p4_list (p4_File *fid, int n);

int p4_refill (void);

void *p4_save_input (void *p);
void *p4_restore_input (void *p);
void p4_source (char **p, int *n);

void p4_skip_delimiter (char del);
int p4_parse (char del, char **p, p4ucell *l);
char *p4_word (char del);

int p4_parseword (char del, char**p, p4ucell *l);
char* p4_hereword (char* p, p4ucell len);

/* dictionary and words lists ============================================== */

p4_Wordl *p4_make_wordlist (void);
void  p4_forget (char *above);
char* p4_forget_word (const char *name, p4cell id, p4code ccode, p4cell what);

int p4_wl_hash (const char *s, int l);
char **p4_topmost (p4_Wordl *w);
char *p4_latest (void);

char *p4_search_wordlist (const char *nm, int l, const p4_Wordl *w);
char *p4_find (const char *nm, int l);
char *p4_tick (p4xt *xt);

FCode (p4_align);			/* in core.c */
FCode (p4_aligned);

char *p4_string_comma (const char *s, int l);
char *p4_parse_comma (char del);
char *p4_word_comma (char del);
char* p4_header (p4code cfa, char flags); /* returns nfa */
char* p4_header_from (p4code cfa, char flags, char* name, int count); /*GD*/

char **p4_name_to_link (const char *p);
char *p4_link_to_name (char **l);
p4xt p4_link_from (char **lnk);
char **p4_to_link (p4xt xt);
#define TO_BODY(C)	((p4cell *)((p4xt)(C) + 1))
#define BODY_FROM(P)	((p4xt)((p4cell *)(P) - 1))
char *p4_to_name (p4xt xt);
p4xt p4_name_from (const char *p);
p4xt p4_runtime (void);
void p4_dot_name (const char *nfa);

void p4_wild_words (const p4_Wordl *wl, const char *pattern, const char *category);
int p4_complete_dictionary (char *in, char *out, int display);

FCode (p4_only);
FCode (p4_also);
FCode (p4_definitions);
FCode (p4_default_order);
FCode (p4_reset_order);

p4_Wordl*
p4_find_vocabulary (char* nm, int nmlen);

p4ucell p4_version_code (void);
void p4_preload_only (void);
void p4_load_llist (void** list, p4_Wordl* wid, int extending);

/* inner and outer interpreter ============================================= */

void p4_run_forth (p4xt xt);
void p4_call (p4xt xt);
void p4_normal_execute (p4xt xt); /* quicker than p4_call but beware...*/
void p4_debug_off (void);

FCode (p4_interpret);
void p4_evaluate (char *p, int n);
void p4_load (p4_File *fid, p4ucell blk);
void p4_load_file (char *fn, int cnt, int blk);
void p4_thru (p4_File *fid, int lo, int hi);
void p4_include_file (p4_File *f);
void p4_included (const char *fn, int n);

/* ABORT and exception handling ============================================ */

void p4_do_abort (void);

int  p4_catch (p4xt xt);		
void p4_throw (int id, ...);	/* accepts -2 addr cnt */
void p4_abortq (const char *fmt, ...); /* accepts printf()-like parameters */

FCode (p4_Q_comp);		/* in misc.c */
FCode (p4_Q_exec);
FCode (p4_Q_pairs);
FCode (p4_store_csp);
FCode (p4_Q_csp);
FCode (p4_Q_loading);
FCode (p4_Q_stack);
void p4_Q_pairs (p4cell n);
void p4_Q_file_open (p4_File *f);

void p4_initialize_system (void);

/* and a few internals */
extern p4_threadP p4_main_threadP; /* distinct for each thread */
void p4_thread_save (p4_threadP);  /* save & load are used on i960 */
void p4_thread_load (p4_threadP);  /* or with MP-emulation of an MT sytem */
#define _p4_thread_save() (p4_thread_save(p4_main_threadP))
#define _p4_thread_load() (p4_thread_load(p4_main_threadP))

#ifdef P4_REGTH
#define _p4_thread_local() (p4TH = p4_main_threadP)
#define _p4_thread_ENTER_(arg_threadP) \
   	register p4_threadP _p4_thread_SAVED_ = p4TH; \
                        	p4TH = arg_threadP;
#define _p4_thread_LEAVE_() p4TH = _p4_thread_SAVED_;
#else
#define _p4_thread_local()
#define _p4_thread_ENTER_(ignored)
#define _p4_thread_LEAVE_()
#endif

/* signal stuff ============================================================ */

void p4_fatal (const char *msg, ...);
void p4_install_signal_handlers (void);
void p4_swap_signals (void);
p4xt   p4_forth_signal (int sig, p4xt xt);
void p4_load_signals (p4_Wordl *wid);

/* more stuff ============================================================= */

extern char	p4_version_string [],
        	p4_copyright_string [],
                p4_license_string [],
                p4_warranty_string [],
                p4_compile_date [],
		p4_compile_time [];

FCode (p4_dot_memory);

#ifdef _P4_SOURCE
#if defined VxWorks || defined SYS_vxworks || defined __target_os_vxworks
#ifndef HAVE_STRDUP
extern char *strdup (const char *s);
#endif
#ifndef HAVE_ACCESS
extern int access (const char *, int);
#endif
/* extern int chdir (const char *); */
extern int close (int);
/* extern int rmdir (const char *); */

#ifndef HAVE_FCNTL
extern int fcntl (int, int, ...);
#endif

#include <fcntl.h>
#include <timers.h>

#define sys_mkdir( dirName, modeBits ) mkdir( (char*)(dirName) )
#else
#define sys_mkdir( dirName, modeBits ) mkdir( dirName, modeBits )
#endif /* VxWorks */

#endif

/*@}*/
#endif
