#ifndef __PFE_LOGGING_H
#define __PFE_LOGGING_H "%full_filespec: logging.h~5.0 %"

/** 
 * -- PFE Log Message System
 *
 *  Copyright (C) Tektronix, Inc. 1998 - 2000. All rights reserved.
 *
 *  @see     GNU LGPL
 *  @author  Tektronix CTE             @(#) %derived_by: guidod %
 *  @version %version: 4.15 %
 *    (%date_modified: Tue Apr 11 15:25:56 2000 %)
 *
 *  @description
 * informational messages, warnings, errors, fatal errors and debug messages.
 *  
 * Only debug messages are intended to be removed from released projects.
 * All others stay in code and wait for their time to appear.
 *
 * These defintions are modelled to be shallow to the macros in K12/logging.h
 *
 */
/*@{*/
/*****************************************************************************/

#ifdef _K12_SOURCE
#  include <K12/logging.h>
#endif

/* this is the log-level at the same time - make 0 to disable all */
#ifndef P4_LOG
#if defined(DEBUG)
#define P4_LOG 0xFFFFFFFF         /* just everything */
#else
#define P4_LOG 0x0E               /* ie. _warn _fail _fatal */
#endif
#endif

#if !P4_LOG
#define P4_LOG_NO_MSG
#endif

/*****************************************************************************/
/* 4 resident predefined message types                                       */
/*****************************************************************************/
#ifdef _K12_SOURCE

#define P4_LOG_INFO     K12_LOG_INFO
#define P4_LOG_WARN     K12_LOG_WARNING
#define P4_LOG_FAIL     K12_LOG_SEVERE
#define P4_LOG_FATAL    K12_LOG_FATAL

#define P4_LOG_MESG     K12_LOG_XINFO_L(0)
#define P4_LOG_NOTE     K12_LOG_XINFO_L(1)
#define P4_LOG_ENTER    K12_LOG_XINFO_L(2)
#define P4_LOG_LEAVE    K12_LOG_XINFO_L(3)

#define P4_LOG_ALL      K12_LOG_ALL

#define P4_LOG_DEFAULT  K12_LOG_DEFAULT
#define P4_LOG_XINFO    K12_LOG_XINFO
#define P4_LOG_DEBUG    K12_LOG_DEBUG
#define P4_LOG_USER     K12_LOG_USER

#else

#define P4_LOG_INFO     (1UL<<0)   
#define P4_LOG_WARN     (1UL<<1)
#define P4_LOG_FAIL     (1UL<<2)
#define P4_LOG_FATAL    (1UL<<3)

#define P4_LOG_MESG     (1UL<<4)
#define P4_LOG_NOTE     (1UL<<5)
#define P4_LOG_ENTER    (1UL<<6)
#define P4_LOG_LEAVE    (1UL<<7)

#define P4_LOG_ALL      0xFFFFFFFFL   /* All Messages */

#define P4_LOG_DEFAULT  0x0000000FL   /* 4 standard groups */
#define P4_LOG_XINFO    0x000000F0L   /* 4 XInfo groups */
#define P4_LOG_DEBUG    0x00FFFF00L   /* 16 Debug groups */
#define P4_LOG_USER     0xFF000000L   /* 8 User groups */

#endif 

/* The following Message groups shall contain informational messages with
** decreasing significance from 1 to 4. The messages shall be used in modules
** where it seems to be sane to tell the user lots of things about what the
** module is actually doing. Like -d option of make or -v option of various
** programs
*/

/*****************************************************************************/
/* Mask makros for dynamically used message types                            */
/*****************************************************************************/

/*  4 XINFO Groups */
#define P4_LOG_XINFO_M(a) (((a)<< 4) & P4_LOG_XINFO)
/* 16 DEBUG Groups */
#define P4_LOG_DEBUG_M(a) (((a)<< 8) & P4_LOG_DEBUG)
/*  8 USER Groups */
#define P4_LOG_USER_M(a)  (((a)<<24) & P4_LOG_USER)

#define P4_LOG_XINFO_L(a) ((0x00000010L << (a)) & P4_LOG_XINFO)
#define P4_LOG_DEBUG_L(a) ((0x00000100L << (a)) & P4_LOG_DEBUG)
#define P4_LOG_USER_L(a)  ((0x01000000L << (a)) & P4_LOG_USER)


/*****************************************************************************/
/* front end makros for generating log messages                              */
/*****************************************************************************/

#ifdef _K12_SOURCE
#  define p4_LogMask k12LogMask
#elif P4_LOG_VAR
#  define p4_LogMask P4_LOG_VAR
#else
#  define p4_LogMask P4_LOG
#endif

#define P4_LogGrpMatch(mask) (((mask) & p4_LogMask))

/* Don't wonder about the misuse of "while" in the following makros. We use
** this construct to avoid unpredictable results when using the makro with
** an "if" statement
*/

#ifndef P4_LOG_NO_MSG

#ifdef _K12_SOURCE

#define P4_LogDumpHex(mask, funcId, s, adr, len, mode) \
  { if(P4_LogGrpMatch(mask)) \
    { k12LogDumpHex(mask, funcId"> "s"\n", adr, len, mode); \
    } \
  } 

#define P4_LogMsg(mask, prefix, funcId, s) \
  { if(P4_LogGrpMatch(mask)) \
      k12LogMsg(mask, prefix"%s> "s"\n", (u32_t)(funcId), 0, 0, 0, 0, 0); \
  } 

#define P4_LogMsg1(mask, prefix, funcId, s, x1) \
  { if(P4_LogGrpMatch(mask)) \
      k12LogMsg(mask, prefix"%s> "s"\n", (u32_t)(funcId), (u32_t)(x1), 0, 0, 0, 0); \
  } 

#define P4_LogMsg2(mask, prefix, funcId, s, x1, x2) \
  { if(P4_LogGrpMatch(mask)) \
      k12LogMsg(mask, prefix"%s> "s"\n", (u32_t)(funcId), (u32_t)(x1), (u32_t)(x2), 0, 0, 0); \
  } 

#define P4_LogMsg3(mask, prefix, funcId, s, x1, x2, x3) \
  { if(P4_LogGrpMatch(mask)) \
      k12LogMsg(mask, prefix"%s> "s"\n", (u32_t)(funcId), (u32_t)(x1), (u32_t)(x2), (u32_t)(x3), 0, 0); \
  } 

#define P4_LogMsg4(mask, prefix, funcId, s, x1, x2, x3, x4) \
  { if(P4_LogGrpMatch(mask)) \
      k12LogMsg(mask, prefix"%s> "s"\n", (u32_t)(funcId), (u32_t)(x1), (u32_t)(x2), (u32_t)(x3),\
        (u32_t)(x4), 0); \
  } 

#define P4_LogMsg5(mask, prefix, funcId, s, x1, x2, x3, x4, x5) \
  { if(P4_LogGrpMatch(mask)) \
      k12LogMsg(mask, prefix"%s> "s"\n", (u32_t)(funcId), (u32_t)(x1), (u32_t)(x2), (u32_t)(x3),\
        (u32_t)(x4), (u32_t)(x5)); \
  } 

#else /* !_K12_SOURCE */
  
#define P4_LogDumpHex(mask, funcId, s, adr, len, mode) \
  { if(P4_LogGrpMatch(mask)) \
    { fprintf (stderr, funcId"> "s" %lx... [%i]\n", *(long*)adr, len); \
    } \
  } 

#define P4_LogMsg(mask, prefix, funcId, s) \
  { if(P4_LogGrpMatch(mask)) \
      fprintf (stderr, prefix"%s> "s"\n", funcId); \
  } 

#define P4_LogMsg1(mask, prefix, funcId, s, x1) \
  { if(P4_LogGrpMatch(mask)) \
      fprintf (stderr, prefix"%s> "s"\n", funcId, x1); \
  } 

#define P4_LogMsg2(mask, prefix, funcId, s, x1, x2) \
  { if(P4_LogGrpMatch(mask)) \
      fprintf (stderr, prefix"%s> "s"\n", funcId, x1, x2); \
  } 

#define P4_LogMsg3(mask, prefix, funcId, s, x1, x2, x3) \
  { if(P4_LogGrpMatch(mask)) \
      fprintf (stderr, prefix"%s> "s"\n", funcId, x1, x2, x3); \
  } 

#define P4_LogMsg4(mask, prefix, funcId, s, x1, x2, x3, x4) \
  { if(P4_LogGrpMatch(mask)) \
      fprintf (stderr, prefix"%s> "s"\n", funcId, x1, x2, x3, x4); \
  } 

#define P4_LogMsg5(mask, prefix, funcId, s, x1, x2, x3, x4, x5) \
  { if(P4_LogGrpMatch(mask)) \
      fprintf (stderr, prefix"%s> "s"\n", funcId, x1, x2, x3, x4, x5); \
  } 

#endif  /* !_K12_SOURCE */

#else

#define P4_LogDumpHex(mask, funcId, s, adr, len, mode)

#define P4_LogMsg(mask, prefix, funcId, s)
#define P4_LogMsg1(mask, prefix, funcId, s, x1)
#define P4_LogMsg2(mask, prefix, funcId, s, x1, x2)
#define P4_LogMsg3(mask, prefix, funcId, s, x1, x2, x3)
#define P4_LogMsg4(mask, prefix, funcId, s, x1, x2, x3, x4)
#define P4_LogMsg5(mask, prefix, funcId, s, x1, x2, x3, x4, x5)

#endif /* K12_LOG_NO_MSG */

/*****************************************************************************/
/* Info Message Makros for one to six parameters           */
/*****************************************************************************/

#ifndef P4_LOG_NO_INFO_MSG

#define P4_info_dump( s, adr, len, mode) \
  P4_LogDumpHex(P4_LOG_INFO, __FUNCTION__, s, adr, len, mode)

#define P4_info(s) \
  P4_LogMsg (P4_LOG_INFO, " _",__FUNCTION__, s)
#define P4_info1(s, x1) \
  P4_LogMsg1(P4_LOG_INFO, " _",__FUNCTION__, s, x1)
#define P4_info2(s, x1, x2) \
  P4_LogMsg2(P4_LOG_INFO, " _",__FUNCTION__, s, x1, x2)
#define P4_info3(s, x1, x2, x3) \
  P4_LogMsg3(P4_LOG_INFO, " _",__FUNCTION__, s, x1, x2, x3)
#define P4_info4(s, x1, x2, x3, x4) \
  P4_LogMsg4(P4_LOG_INFO, " _",__FUNCTION__, s, x1, x2, x3, x4)
#define P4_info5(s, x1, x2, x3, x4, x5) \
  P4_LogMsg5(P4_LOG_INFO, " _",__FUNCTION__, s, x1, x2, x3, x4, x5)
  
#else

#define P4_info_dump( s, adr, len, mode)

#define P4_info( s)
#define P4_info1( s, x1)
#define P4_info2( s, x1, x2)
#define P4_info3( s, x1, x2, x3)
#define P4_info4( s, x1, x2, x3, x4)
#define P4_info5( s, x1, x2, x3, x4, x5)

#endif /* P4_LOG_NO_INFO_MSG */

/*****************************************************************************/
/* Warning Message Makros for one to six parameters                          */
/*****************************************************************************/

#ifndef P4_LOG_NO_WARN_MSG

#define P4_warn_dump( s, adr, len, mode) \
  P4_LogDumpHex(P4_LOG_WARNING, __FUNCTION__, s, adr, len, mode)

#define P4_warn( s) \
  P4_LogMsg (P4_LOG_WARN, "w!",__FUNCTION__, s)
#define P4_warn1( s, x1) \
  P4_LogMsg1(P4_LOG_WARN, "w!",__FUNCTION__, s, x1)
#define P4_warn2( s, x1, x2) \
  P4_LogMsg2(P4_LOG_WARN, "w!",__FUNCTION__, s, x1, x2)
#define P4_warn3( s, x1, x2, x3) \
  P4_LogMsg3(P4_LOG_WARN, "w!",__FUNCTION__, s, x1, x2, x3)
#define P4_warn4( s, x1, x2, x3, x4) \
  P4_LogMsg4(P4_LOG_WARN, "w!",__FUNCTION__, s, x1, x2, x3, x4)
#define P4_warn5( s, x1, x2, x3, x4, x5) \
  P4_LogMsg5(P4_LOG_WARN, "w!",__FUNCTION__, s, x1, x2, x3, x4, x5)
  
#else

#define P4_warn_dump( s, adr, len, mode)

#define P4_warn( s)
#define P4_warn1( s, x1)
#define P4_warn2( s, x1, x2)
#define P4_warn3( s, x1, x2, x3)
#define P4_warn4( s, x1, x2, x3, x4)
#define P4_warn5( s, x1, x2, x3, x4, x5)

#endif /* P4_LOG_NO_WARN_MSG */

/*****************************************************************************/
/* Severe Error Message Makros for one to six parameters         */
/*****************************************************************************/

#ifndef P4_LOG_NO_FAIL_MSG

#define P4_fail_dump( s, adr, len, mode) \
  P4_LogDumpHex(P4_LOG_FAIL, __FUNCTION__, s, adr, len, mode)

#define P4_fail( s) \
  P4_LogMsg (P4_LOG_FAIL, "F!",__FUNCTION__, s)
#define P4_fail1( s, x1) \
  P4_LogMsg1(P4_LOG_FAIL, "F!",__FUNCTION__, s, x1)
#define P4_fail2( s, x1, x2) \
  P4_LogMsg2(P4_LOG_FAIL, "F!",__FUNCTION__, s, x1, x2)
#define P4_fail3( s, x1, x2, x3) \
  P4_LogMsg3(P4_LOG_FAIL, "F!",__FUNCTION__, s, x1, x2, x3)
#define P4_fail4( s, x1, x2, x3, x4) \
  P4_LogMsg4(P4_LOG_FAIL, "F!",__FUNCTION__, s, x1, x2, x3, x4)
#define P4_fail5( s, x1, x2, x3, x4, x5) \
  P4_LogMsg5(P4_LOG_FAIL, "F!",__FUNCTION__, s, x1, x2, x3, x4, x5)

#else

#define P4_fail_dump( s, adr, len, mode)

#define P4_fail( s)
#define P4_fail1( s, x1)
#define P4_fail2( s, x1, x2)
#define P4_fail3( s, x1, x2, x3)
#define P4_fail4( s, x1, x2, x3, x4)
#define P4_fail5( s, x1, x2, x3, x4, x5)

#endif /* P4_LOG_NO_SEVERE_MSG */

/*****************************************************************************/
/* Fatal Error Message Makros for one to six parameters          */
/*****************************************************************************/

#ifndef P4_LOG_NO_FATAL_MSG

#define P4_fatal_dump(s, adr, len, mode) \
  P4_LogDumpHex(P4_LOG_FATAL, __FUNCTION__, s, adr, len, mode)

#define P4_fatal(s) \
  P4_LogMsg (P4_LOG_FATAL, "X!",__FUNCTION__, s)
#define P4_fatal1(s, x1) \
  P4_LogMsg1(P4_LOG_FATAL, "X!",__FUNCTION__, s, x1)
#define P4_fatal2(s, x1, x2) \
  P4_LogMsg2(P4_LOG_FATAL, "X!",__FUNCTION__, s, x1, x2)
#define P4_fatal3(s, x1, x2, x3) \
  P4_LogMsg3(P4_LOG_FATAL, "X!",__FUNCTION__, s, x1, x2, x3)
#define P4_fatal4(s, x1, x2, x3, x4) \
  P4_LogMsg4(P4_LOG_FATAL, "X!",__FUNCTION__, s, x1, x2, x3, x4)
#define P4_fatal5(s, x1, x2, x3, x4, x5) \
  P4_LogMsg5(P4_LOG_FATAL, "X!",__FUNCTION__, s, x1, x2, x3, x4, x5)

#else

#define P4_fatal_dump(s, adr, len, mode)

#define P4_fatal(s)
#define P4_fatal1(s, x1)
#define P4_fatal2(s, x1, x2)
#define P4_fatal3(s, x1, x2, x3)
#define P4_fatal4(s, x1, x2, x3, x4)
#define P4_fatal5(s, x1, x2, x3, x4, x5)

#endif /* P4_LOG_NO_FATAL_MSG */

/*****************************************************************************/
/* Function Enter/Leave Message Makros for one to six parameters          */
/*****************************************************************************/

#ifndef P4_LOG_NO_XINFO_MSG

#define P4_mesg_dump( s, adr, len, mode) \
  P4_LogDumpHex(P4_LOG_MESG, __FUNCTION__, s, adr, len, mode)

#define P4_mesg( s) \
  P4_LogMsg (P4_LOG_MESG, "*_",__FUNCTION__, s)
#define P4_mesg1( s, x1) \
  P4_LogMsg1(P4_LOG_MESG, "*_",__FUNCTION__, s, x1)
#define P4_mesg2( s, x1, x2) \
  P4_LogMsg2(P4_LOG_MESG, "*_",__FUNCTION__, s, x1, x2)
#define P4_mesg3( s, x1, x2, x3) \
  P4_LogMsg3(P4_LOG_MESG, "*_",__FUNCTION__, s, x1, x2, x3)
#define P4_mesg4( s, x1, x2, x3, x4) \
  P4_LogMsg4(P4_LOG_MESG, "*_",__FUNCTION__, s, x1, x2, x3, x4)
#define P4_mesg5( s, x1, x2, x3, x4, x5) \
  P4_LogMsg5(P4_LOG_MESG, "*_",__FUNCTION__, s, x1, x2, x3, x4, x5)

#else

#define P4_mesg_dump( s, adr, len, mode)

#define P4_mesg( s)
#define P4_mesg1( s, x1)
#define P4_mesg2( s, x1, x2)
#define P4_mesg3( s, x1, x2, x3)
#define P4_mesg4( s, x1, x2, x3, x4)
#define P4_mesg5( s, x1, x2, x3, x4, x5)

#endif /* P4_LOG_NO_XINFO_MSG - mesg* part */

#ifndef P4_LOG_NO_XINFO_MSG

#define P4_note_dump( s, adr, len, mode) \
  P4_LogDumpHex(P4_LOG_NOTE, __FUNCTION__, s, adr, len, mode)

#define P4_note( s) \
  P4_LogMsg (P4_LOG_NOTE, "!_",__FUNCTION__, s)
#define P4_note1( s, x1) \
  P4_LogMsg1(P4_LOG_NOTE, "!_",__FUNCTION__, s, x1)
#define P4_note2( s, x1, x2) \
  P4_LogMsg2(P4_LOG_NOTE, "!_",__FUNCTION__, s, x1, x2)
#define P4_note3( s, x1, x2, x3) \
  P4_LogMsg3(P4_LOG_NOTE, "!_",__FUNCTION__, s, x1, x2, x3)
#define P4_note4( s, x1, x2, x3, x4) \
  P4_LogMsg4(P4_LOG_NOTE, "!_",__FUNCTION__, s, x1, x2, x3, x4)
#define P4_note5( s, x1, x2, x3, x4, x5) \
  P4_LogMsg5(P4_LOG_NOTE, "!_",__FUNCTION__, s, x1, x2, x3, x4, x5)

#else

#define P4_note_dump( s, adr, len, mode)

#define P4_note( s)
#define P4_note1( s, x1)
#define P4_note2( s, x1, x2)
#define P4_note3( s, x1, x2, x3)
#define P4_note4( s, x1, x2, x3, x4)
#define P4_note5( s, x1, x2, x3, x4, x5)

#endif /* P4_LOG_NO_XINFO_MSG - note* part */

#ifndef P4_LOG_NO_XINFO_MSG

#define P4_enter_dump( s, adr, len, mode) \
  P4_LogDumpHex(P4_LOG_ENTER, __FUNCTION__, s, adr, len, mode)

#define P4_enter( s) \
  P4_LogMsg (P4_LOG_ENTER, "<\\",__FUNCTION__, s)
#define P4_enter1( s, x1) \
  P4_LogMsg1(P4_LOG_ENTER, "<\\",__FUNCTION__, s, x1)
#define P4_enter2( s, x1, x2) \
  P4_LogMsg2(P4_LOG_ENTER, "<\\",__FUNCTION__, s, x1, x2)
#define P4_enter3( s, x1, x2, x3) \
  P4_LogMsg3(P4_LOG_ENTER, "<\\",__FUNCTION__, s, x1, x2, x3)
#define P4_enter4( s, x1, x2, x3, x4) \
  P4_LogMsg4(P4_LOG_ENTER, "<\\",__FUNCTION__, s, x1, x2, x3, x4)
#define P4_enter5( s, x1, x2, x3, x4, x5) \
  P4_LogMsg5(P4_LOG_ENTER, "<\\",__FUNCTION__, s, x1, x2, x3, x4, x5)

#else

#define P4_enter_dump( s, adr, len, mode)

#define P4_enter( s)
#define P4_enter1( s, x1)
#define P4_enter2( s, x1, x2)
#define P4_enter3( s, x1, x2, x3)
#define P4_enter4( s, x1, x2, x3, x4)
#define P4_enter5( s, x1, x2, x3, x4, x5)

#endif /* P4_LOG_NO_XINFO_MSG - enter* part */

#ifndef P4_LOG_NO_XINFO_MSG

#define P4_leave_dump( s, adr, len, mode) \
  P4_LogDumpHex(P4_LOG_LEAVE, __FUNCTION__, s, adr, len, mode)

#define P4_leave( s) \
  P4_LogMsg (P4_LOG_LEAVE, "</",__FUNCTION__, s)
#define P4_leave1( s, x1) \
  P4_LogMsg1(P4_LOG_LEAVE, "</",__FUNCTION__, s, x1)
#define P4_leave2( s, x1, x2) \
  P4_LogMsg2(P4_LOG_LEAVE, "</",__FUNCTION__, s, x1, x2)
#define P4_leave3( s, x1, x2, x3) \
  P4_LogMsg3(P4_LOG_LEAVE, "</",__FUNCTION__, s, x1, x2, x3)
#define P4_leave4( s, x1, x2, x3, x4) \
  P4_LogMsg4(P4_LOG_LEAVE, "</",__FUNCTION__, s, x1, x2, x3, x4)
#define P4_leave5( s, x1, x2, x3, x4, x5) \
  P4_LogMsg5(P4_LOG_LEAVE, "</",__FUNCTION__, s, x1, x2, x3, x4, x5)

#else

#define P4_leave_dump( s, adr, len, mode)

#define P4_leave( s)
#define P4_leave1( s, x1)
#define P4_leave2( s, x1, x2)
#define P4_leave3( s, x1, x2, x3)
#define P4_leave4( s, x1, x2, x3, x4)
#define P4_leave5( s, x1, x2, x3, x4, x5)

#endif /* P4_LOG_NO_XINFO_MSG - leave* part */

#ifndef P4_LOG_NO_XINFO_MSG

#define P4_return_dump( s, adr, len, mode) { \
  P4_LogDumpHex(P4_LOG_LEAVE, __FUNCTION__, s, adr, len, mode); \
  return ; }

#define P4_return( s) { \
  P4_LogMsg (P4_LOG_LEAVE, "</",__FUNCTION__, s); return; }
#define P4_return1( s, x1) { \
  P4_LogMsg1(P4_LOG_LEAVE, "</",__FUNCTION__, s, x1); return; }
#define P4_return2( s, x1, x2) { \
  P4_LogMsg2(P4_LOG_LEAVE, "</",__FUNCTION__, s, x1, x2); return; }
#define P4_return3( s, x1, x2, x3) { \
  P4_LogMsg3(P4_LOG_LEAVE, "</",__FUNCTION__, s, x1, x2, x3); return; }
#define P4_return4( s, x1, x2, x3, x4) { \
  P4_LogMsg4(P4_LOG_LEAVE, "</",__FUNCTION__, s, x1, x2, x3, x4); return; }
#define P4_return5( s, x1, x2, x3, x4, x5) { \
  P4_LogMsg5(P4_LOG_LEAVE, "</",__FUNCTION__, s, x1, x2, x3, x4, x5); return; }

#else

#define P4_return_dump( s, adr, len, mode) return

#define P4_return( s) return
#define P4_return1( s, x1) return
#define P4_return2( s, x1, x2) return
#define P4_return3( s, x1, x2, x3) return
#define P4_return4( s, x1, x2, x3, x4) return
#define P4_return5( s, x1, x2, x3, x4, x5) return

#endif /* P4_LOG_NO_XINFO_MSG - return* part */

#ifndef P4_LOG_NO_XINFO_MSG

#define P4_returns_dump( a, s, adr, len, mode) { \
  P4_LogDumpHex(P4_LOG_LEAVE, __FUNCTION__, s, adr, len, mode); \
  return a ; }

#define P4_returns( a, s) { \
  P4_LogMsg (P4_LOG_LEAVE, "</",__FUNCTION__, s); return a; }
#define P4_returns1( a, s, x1) { \
  P4_LogMsg1(P4_LOG_LEAVE, "</",__FUNCTION__, s, x1); return a; }
#define P4_returns2( a, s, x1, x2) { \
  P4_LogMsg2(P4_LOG_LEAVE, "</",__FUNCTION__, s, x1, x2); return a; }
#define P4_returns3( a, s, x1, x2, x3) { \
  P4_LogMsg3(P4_LOG_LEAVE, "</",__FUNCTION__, s, x1, x2, x3); return a; }
#define P4_returns4( a, s, x1, x2, x3, x4) { \
  P4_LogMsg4(P4_LOG_LEAVE, "</",__FUNCTION__, s, x1, x2, x3, x4); return a; }
#define P4_returns5( a, s, x1, x2, x3, x4, x5) { \
  P4_LogMsg5(P4_LOG_LEAVE, "</",__FUNCTION__, s, x1, x2, x3, x4, x5); return a; }

#else

#define P4_returns_dump( s, adr, len, mode) return

#define P4_returns( a, s) return a
#define P4_returns1( a, s, x1) return a
#define P4_returns2( a, s, x1, x2) return a
#define P4_returns3( a, s, x1, x2, x3) return a
#define P4_returns4( a, s, x1, x2, x3, x4) return a
#define P4_returns5( a, s, x1, x2, x3, x4, x5) return a

#endif /* P4_LOG_NO_XINFO_MSG - returns* part */

/*****************************************************************************/
/* Debug Messages for one to six parameters            */
/*****************************************************************************/

#ifndef P4_LOG_NO_DEBUG_MSG

#define P4_debug_dump(mask, s, adr, len, mode) \
  P4_LogDumpHex(P4_LOG_DEBUG_L(mask), __FUNCTION__, s, adr, len, mode)

#define P4_debug(mask, s) \
  P4_LogMsg (P4_LOG_DEBUG_L(mask), "..",__FUNCTION__, s)
#define P4_debug1(mask, s, x1) \
  P4_LogMsg1(P4_LOG_DEBUG_L(mask), "..",__FUNCTION__, s, x1)
#define P4_debug2(mask, s, x1, x2) \
  P4_LogMsg2(P4_LOG_DEBUG_L(mask), "..",__FUNCTION__, s, x1, x2)
#define P4_debug3(mask, s, x1, x2, x3) \
  P4_LogMsg3(P4_LOG_DEBUG_L(mask), "..",__FUNCTION__, s, x1, x2, x3)
#define P4_debug4(mask, s, x1, x2, x3, x4) \
  P4_LogMsg4(P4_LOG_DEBUG_L(mask), "..",__FUNCTION__, s, x1, x2, x3, x4)
#define P4_debug5(mask, s, x1, x2, x3, x4, x5) \
  P4_LogMsg5(P4_LOG_DEBUG_L(mask), "..",__FUNCTION__, s, x1, x2, x3, x4, x5)

#else

#define P4_debug_dump(mask, s, adr, len, mode)

#define P4_debug(mask, s)
#define P4_debug1(mask, s, x1)
#define P4_debug2(mask, s, x1, x2)
#define P4_debug3(mask, s, x1, x2, x3)
#define P4_debug4(mask, s, x1, x2, x3, x4)
#define P4_debug5(mask, s, x1, x2, x3, x4, x5)

#endif /* P4_LOG_NO_DEBUG_MSG */

/*****************************************************************************/
/* Debug Messages for one to six parameters            */
/*****************************************************************************/

#ifndef P4_LOG_NO_MSG

#define P4_log_dump(mask, s, adr, len, mode) \
  P4_LogDumpHex(P4_LOG_USER_L(mask), __FUNCTION__, s, adr, len, mode)

#define P4_log(mask, s) \
  P4_LogMsg (P4_LOG_USER_L(mask), "  ",__FUNCTION__, s)
#define P4_log1(mask, s, x1) \
  P4_LogMsg1(P4_LOG_USER_L(mask), "  ",__FUNCTION__, s, x1)
#define P4_log2(mask, s, x1, x2) \
  P4_LogMsg2(P4_LOG_USER_L(mask), "  ",__FUNCTION__, s, x1, x2)
#define P4_log3(mask, s, x1, x2, x3) \
  P4_LogMsg3(P4_LOG_USER_L(mask), "  ",__FUNCTION__, s, x1, x2, x3)
#define P4_log4(mask, s, x1, x2, x3, x4) \
  P4_LogMsg4(P4_LOG_USER_L(mask), "  ",__FUNCTION__, s, x1, x2, x3, x4)
#define P4_log5(mask, s, x1, x2, x3, x4, x5) \
  P4_LogMsg5(P4_LOG_USER_L(mask), "  ",__FUNCTION__, s, x1, x2, x3, x4, x5)

#else

#define P4_log_dump(mask, s, adr, len, mode)

#define P4_log(mask, s)
#define P4_log1(mask, s, x1)
#define P4_log2(mask, s, x1, x2)
#define P4_log3(mask, s, x1, x2, x3)
#define P4_log4(mask, s, x1, x2, x3, x4)
#define P4_log5(mask, s, x1, x2, x3, x4, x5)

#endif /* P4_LOG_NO_DEBUG_MSG */

/******************************************************************************
** Makros instead of Real Prototypes to remove the calls with P4_LOG_NO_MSG
******************************************************************************/
#if defined(P4_LOG_NO_MSG) || ! defined(_K12_SOURCE)

#define P4_LogInit(logFile, maxMsg, logMode)
#define P4_LogOptionSet(msgMask, msgFilter, tId, xflg)
#define P4_LogOptionSetN(msgMask, msgFilter, tname, xflg)
#define P4_LogOptionGet(pMsgMask, msgFilter, ptId, pxflg)
#define P4_LogOptionShow()

#else

#define P4_LogInit(logFile, maxMsg, logMode) \
  k12LogInit(logFile, maxMsg, logMode)
#define P4_LogOptionSet(msgMask, msgFilter, tId, xflg) \
  k12LogOptionSet(msgMask, msgFilter, tId, xflg)
#define P4_LogOptionSetN(msgMask, msgFilter, tname, xflg) \
  k12LogOptionSetN(msgMask, msgFilter, tname, xflg)
#define P4_LogOptionGet(pMsgMask, msgFilter, ptId, pxflg) \
  k12LogOptionGet(pMsgMask, msgFilter, ptId, pxflg)
#define P4_LogOptionShow() \
  k12LogOptionShow()
  
#endif /* P4_LOG_NO_MSG */

/*@}*/
#endif
